/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/types.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <soc.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>

#include <bluetooth/services/lbs.h>

#include <zephyr/settings/settings.h>

#include <dk_buttons_and_leds.h>

#include <zephyr/sys/reboot.h>

#include <hal/nrf_power.h>

// Additions for DFU
#include <zephyr/mgmt/mcumgr/smp_bt.h>
#include "os_mgmt/os_mgmt.h"
#include "img_mgmt/img_mgmt.h"
// End Additions for DFU

#define DEVICE_NAME             CONFIG_BT_DEVICE_NAME		// prj.conf
#define DEVICE_NAME_LEN         (sizeof(DEVICE_NAME) - 1)


#define RUN_STATUS_LED          DK_LED1
#define CON_STATUS_LED          DK_LED2
#define RUN_LED_BLINK_INTERVAL  1000

#define USER_LED                DK_LED3

#define USER_BUTTON             DK_BTN1_MSK

// Advertisting data
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
};

// Scan response data.  This is the Bluetooth SIG SMP UUID 
static const struct bt_data sd[] = {
//	BT_DATA_BYTES(BT_DATA_UUID128_ALL, BT_UUID_LBS_VAL),
	BT_DATA_BYTES(BT_DATA_UUID128_ALL,
		      0x84, 0xaa, 0x60, 0x74, 0x52, 0x8a, 0x8b, 0x86,
		      0xd3, 0x4c, 0xb7, 0x1d, 0x1d, 0xdc, 0x53, 0x8d),
};

static void connected(struct bt_conn *conn, uint8_t err)
{
	if (err) {
		printk("Connection failed (err %u)\n", err);
		return;
	}

	printk("Connected\n");

	dk_set_led_on(CON_STATUS_LED);
}

static void disconnected(struct bt_conn *conn, uint8_t reason)
{
	printk("Disconnected (reason %u)\n", reason);

	dk_set_led_off(CON_STATUS_LED);
}


BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected        = connected,
	.disconnected     = disconnected,
};

#define SW0_NODE	DT_ALIAS(sw0)

static const struct gpio_dt_spec button = GPIO_DT_SPEC_GET(SW0_NODE, gpios);

void main(void)
{
	int blink_status = 0;
	int err;

	int ret;
	bool val;

	uint8_t value;

	if (!device_is_ready(button.port)) {
		return;
	}

	ret = gpio_pin_configure_dt(&button, GPIO_INPUT);
	if (ret < 0) {
		return;
	}

    val = gpio_pin_get_dt(&button);
	if(val){
		printk("Release the button\n");
	}
	k_sleep(K_MSEC(2000));


	if(val){

		nrf_power_gpregret2_set(NRF_POWER, 0x0B);

		printk("button press detected \n");	

		sys_reboot(SYS_REBOOT_WARM);
	}else{
		
		value = nrf_power_gpregret2_get(NRF_POWER);

		if(value == 0x0B){
			printk("Start the bootloader!!!\n");
			printk("value = 0x%x\n", value);
		}else{
			printk("Do not start the bootloader!!!\n");
			printk("value = 0x%x\n", value);
		}
	}

	if(value == 0x0B){
		nrf_power_gpregret2_set(NRF_POWER, 0x00);
		k_sleep(K_MSEC(50));
		value = nrf_power_gpregret2_get(NRF_POWER);

		printk("Starting Bootloader\n");
		printk("value = 0x%x\n", value);

 		err = dk_leds_init();
		if (err) {
			printk("LEDs init failed (err %d)\n", err);
			return;
		}

		printk("build time: " __DATE__ " " __TIME__ "\n");
		os_mgmt_register_group();
		img_mgmt_register_group();
		smp_bt_register();	

		err = bt_enable(NULL);
		if (err) {
			printk("Bluetooth init failed (err %d)\n", err);
			return;
		}

		printk("Bluetooth initialized\n");

		err = bt_le_adv_start(BT_LE_ADV_CONN, ad, ARRAY_SIZE(ad),
					sd, ARRAY_SIZE(sd));
		if (err) {
			printk("Advertising failed to start (err %d)\n", err);
			return;
		}

		printk("Advertising successfully started\n");

		for (;;) {
			dk_set_led(RUN_STATUS_LED, (++blink_status) % 2);
			k_sleep(K_MSEC(RUN_LED_BLINK_INTERVAL));
		}
	}else{
		printk("\nVersion 4\n");
	}
}
